
#include "../mdk/mdk.h"
#include <windows.h>
#include <math.h>

#pragma optimize ("awy", on)

CMachineParameter const paraRecLength =
{
	pt_word,
	"Length",
	"Recording Length in Ticks",
	0,
	0xFFFE,
	0xFFFF,
	MPF_STATE,
	0
};

CMachineParameter const paraWave =
{
	pt_byte,
	"Wave",
	"Which wave to record over?",
	0,
	0xC9,
	0xFF,
	MPF_STATE,
	0
};

CMachineParameter const paraRecMode =
{
	pt_byte,
	"Mode",
	"Recording Mode",
	0,
	6,
	0xFF,
	MPF_STATE,
	0
};

CMachineParameter const paraRecordingVol =
{
	pt_byte,
	"Volume",
	"Recording Volume",
	0,
	0xFE,
	0xFF,
	MPF_STATE,
	127
};

CMachineParameter const paraTriggerStart =
{
	pt_switch,
	"Record Trigger Start",
	"Record Trigger Start (1=on, 0=off)",
	-1,
	-1,
	SWITCH_NO,
	0,
	0
};

CMachineParameter const *pParameters[] = {
	&paraRecLength,
	&paraWave,
	&paraRecMode,
	&paraRecordingVol,
	&paraTriggerStart
	};


CMachineAttribute const attrMIDIChannel = {
	"MIDI Channel",
	0,
	16,
	0
};

CMachineAttribute const attrMIDITrigger = {
	"MIDI Trigger (0=no 1=sustain 2=1on1off)",
	0,
	2,
	2
};

CMachineAttribute const attrMIDINoteOn = {
	"MIDI Note Trigger On",
	0,
	120,
	0
};

CMachineAttribute const attrMIDINoteOff = {
	"MIDI Note Trigger Off",
	0,
	120,
	0
};

CMachineAttribute const attrMIDIMaxLen = {
	"MIDI Max Length in Ticks",
	0,
	65534,
	0
};

CMachineAttribute const attrMIDIRecordable = {
	"MIDI Recording Enabled",
	0,
	1,
	1
};

CMachineAttribute const attrStartGateLevel = {
	"Start Gate Level",
	-90,
	0,
	-90
};

CMachineAttribute const attrStartGateOn = {
	"Start Gate Enabled",
	0,
	1,
	0
};

CMachineAttribute const *pAttributes[] = {
	&attrMIDIChannel,
	&attrMIDITrigger,
	&attrMIDINoteOn,
	&attrMIDINoteOff,
	&attrMIDIMaxLen,
	&attrMIDIRecordable,
	&attrStartGateLevel,
	&attrStartGateOn
};

#pragma pack(1)

class gvals
{
public:
	word length;
	byte wave;
	byte recmode;
	byte recvolume;
	byte trigger;
};

class avals
{
public:
	int midichannel;
	int miditrigger;
	int midinoteon;
	int midinoteoff;
	int midimaxlen;
	int midirecord;
	int startGateLevel;
	int startGateOn;
};

#pragma pack()


CMachineInfo const MacInfo =
{
	MT_EFFECT,
	MI_VERSION,
	MIF_DOES_INPUT_MIXING | MIF_NO_OUTPUT,
	0,										// min tracks
	0,										// max tracks
	5,										// numGlobalParameters
	0,										// numTrackParameters
	pParameters,
	8,
	pAttributes,
	"CnG Recorder",					// name
	"CnG Recorder",						// short name
	"Edward L. Blake (minor improvements and debugging by Geoffroy Montel)",						// author
	"Set MIDI Note On\nSet MIDI Note Off\n&About..."
};


class miex : public CMDKMachineInterfaceEx { };

class mi : public CMDKMachineInterface
{
public:
	mi();
	virtual ~mi();

	virtual void Tick();

	virtual void MDKInit(CMachineDataInput * const pi);
	virtual bool MDKWork(float *psamples, int numsamples, int const mode);
	virtual bool MDKWorkStereo(float *psamples, int numsamples, int const mode);
	virtual void Command(int const i);
	virtual void AttributesChanged();

	virtual void MDKSave(CMachineDataOutput * const po);
	virtual char const *DescribeValue(int const param, int const value);

	virtual int AllocateTrack(CSequence *pseq, int note);
	virtual void MidiNoteOff(int c, CSequence *pseq, int notedelay);
	virtual void MidiNote(int const channel, int const value, int const velocity);

public:
	virtual CMDKMachineInterfaceEx *GetEx() { return &ex; }
	virtual void OutputModeChanged(bool stereo) {}


public:
	miex ex;

public:
	CMachine *ThisMachine;

public:
	float currentrecvol;
	float recvol;
	int recordnow;
	int wavetouse;
	int waveinuse;
	int inputmode;
	int currentinputmode;
	int muteme;
	int numofsamples;
	int currentnumofsamples;
	int currentnum;

	int setmidion;
	int setmidioff;

	// correction Geoffroy 01/05/2003
	float startGateLevel;
	int gateActive;
	int numsamplesabovegate;
	// fin correction Geoffroy 01/05/2003

public:
	gvals gval;
	avals aval;

};

mi::mi() {
	GlobalVals = &gval;
	AttrVals = (int *)&aval;
}
mi::~mi() { }

void mi::MDKInit(CMachineDataInput * const pi)
{
	ThisMachine = pCB->GetThisMachine();

	SetOutputMode( true );

	currentrecvol = 0.0f;
	recordnow = 0;
	waveinuse = 0;
	currentinputmode = 0;
	muteme = 0;
	currentnumofsamples = 0;
	currentnum = 0;
	setmidion = 0;
	setmidioff = 0;
	inputmode = 0;
	numofsamples = 0;
	wavetouse = 0;
	recvol = 1.0f;

	// correction Geoffroy 01/05/2003
	startGateLevel = (float) pow (10.0f,(float)aval.startGateLevel / 40.0f); // Amp = pow( 10.0, ( dB / 40.0 ) );
	gateActive = 0;
	if (aval.startGateOn == 1) {
		gateActive = 1;
		numsamplesabovegate = 0;
	}
	// fin correction Geoffroy 01/05/2003

	if (gval.recmode != 0xFF) inputmode = gval.recmode;
	if (gval.length != 0xFFFF) numofsamples = gval.length * pMasterInfo->SamplesPerTick;
	if (gval.wave != 0xFF) wavetouse = gval.wave;
	if (gval.recvolume != 0xFF) recvol = gval.recvolume/127.0f;

}

void mi::AttributesChanged() {
	// correction Geoffroy 01/05/2003
	startGateLevel = (float) pow (10.0f,(float)aval.startGateLevel / 40.0f); // Amp = pow( 10.0, ( dB / 40.0 ) );
	if (aval.startGateOn == 1) {
		gateActive = 1;
		numsamplesabovegate = 0;
	}
	// fin correction Geoffroy 01/05/2003
}


void mi::MDKSave(CMachineDataOutput * const po) { }

void mi::Tick() {
	int wavebuffer;


	if (gval.recmode != 0xFF) inputmode = gval.recmode;
	if (gval.length != 0xFFFF) numofsamples = gval.length * pMasterInfo->SamplesPerTick;
	if (gval.wave != 0xFF) wavetouse = gval.wave;
	if (gval.recvolume != 0xFF) recvol = gval.recvolume/127.0f;
	if (gval.trigger != SWITCH_NO) {
		if (gval.trigger == 1) {
			if (wavetouse != 0) {
				if (numofsamples > 0) {
					if (wavetouse == 201) {
						if (pCB->GetWave(199) != NULL) return;
						if (pCB->GetWave(200) != NULL) return;
						wavebuffer = pCB->GetFreeWave();
						if ((wavebuffer < 0) || (wavebuffer > 198)) return;
						waveinuse = wavebuffer;

					} else {
						waveinuse = wavetouse;
					};
					currentrecvol = recvol;
					currentinputmode = inputmode;
					currentnumofsamples = numofsamples;
					currentnum = 0;
					recordnow = 1;
					if (aval.startGateOn == 1) {
						gateActive = 1;
						numsamplesabovegate = 0;
					}

					switch (currentinputmode) {
					case 0:
						// Stereo Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo") == true) {
							currentnumofsamples *= 2;
							CWaveInfo const *mewav = pCB->GetWave(waveinuse);
							int *mewavefl = (int *)&mewav->Flags;
							*mewavefl = *mewavefl | WF_STEREO;

							CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
							int *mewavens = (int *)&mewav2->numSamples;
							*mewavens = currentnumofsamples / 2;
							int *mewavele = (int *)&mewav2->LoopEnd;
							*mewavele = currentnumofsamples / 2;

						} else {
							recordnow = 0;
						};
						break;
					case 1:
						// Stereo Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (Inv. Chan.)") == true) {
							currentnumofsamples *= 2;
							CWaveInfo const *mewav = pCB->GetWave(waveinuse);
							int *mewavefl = (int *)&mewav->Flags;
							*mewavefl = *mewavefl | WF_STEREO;

							CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
							int *mewavens = (int *)&mewav2->numSamples;
							*mewavens = currentnumofsamples / 2;
							int *mewavele = (int *)&mewav2->LoopEnd;
							*mewavele = currentnumofsamples / 2;

						} else {
							recordnow = 0;
						};
						break;
					case 2:
						// Stereo Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-L)") == true) {
							currentnumofsamples *= 2;
							CWaveInfo const *mewav = pCB->GetWave(waveinuse);
							int *mewavefl = (int *)&mewav->Flags;
							*mewavefl = *mewavefl | WF_STEREO;

							CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
							int *mewavens = (int *)&mewav2->numSamples;
							*mewavens = currentnumofsamples / 2;
							int *mewavele = (int *)&mewav2->LoopEnd;
							*mewavele = currentnumofsamples / 2;

						} else {
							recordnow = 0;
						};
						break;
					case 3:
						// Stereo Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-R)") == true) {
							currentnumofsamples *= 2;
							CWaveInfo const *mewav = pCB->GetWave(waveinuse);
							int *mewavefl = (int *)&mewav->Flags;
							*mewavefl = *mewavefl | WF_STEREO;

							CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
							int *mewavens = (int *)&mewav2->numSamples;
							*mewavens = currentnumofsamples / 2;
							int *mewavele = (int *)&mewav2->LoopEnd;
							*mewavele = currentnumofsamples / 2;

						} else {
							recordnow = 0;
						};
						break;
					case 4:
						// Mono Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono") == true) {

						} else {
							recordnow = 0;
						};
						break;
					case 5:
						// Mono Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-L)") == true) {

						} else {
							recordnow = 0;
						};
						break;
					case 6:
						// Mono Shit
						if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-R)") == true) {

						} else {
							recordnow = 0;
						};
						break;
					default:
						recordnow = 0;
						break;
					}
				};
			};
		} else {
			recordnow = 0;
		};
	};
}

bool mi::MDKWork(float *psamples, int numsamples, int const mode)
{
	return false;
}

bool mi::MDKWorkStereo(float *psamples, int numsamples, int const mode)
{
	float inL, inR;
	int i;
	unsigned int u;

	// correction Geoffroy 01/05/2003
	if (gateActive == 1) {
		for( i=0; i<numsamples*2; i++ ) {
			if (psamples[i] > startGateLevel) {
				numsamplesabovegate++;
				// wait a bit before activating (buzz tends to drop some samples)
				if (numsamplesabovegate>1) {
					gateActive = 0;
					break;
				}
			}
		}
	}
	// fin correction Geoffroy 01/05/2003

	if (recordnow == 1 && gateActive == 0) {	// correction Geoffroy 1/5/2003
		CWaveLevel const *pLeva = pCB->GetWaveLevel(waveinuse,0);

		// correction Geoffroy 21/02/2003
	    if (mode==WM_WRITE || mode==WM_NOIO) {
			for( i=0; i<numsamples*2; i++ ) {
				if (currentnum < currentnumofsamples) {
					u = currentnum;
					pLeva->pSamples[u] = 0;
					currentnum++;
				}
				if (currentnum < currentnumofsamples) {
					u = currentnum;
					pLeva->pSamples[u] = 0;
					currentnum++;
				}
				i++;
			}
		}
		else {
		// end correction Geoffroy 21/02/2003

			switch (currentinputmode)
			{
			case 0:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i] * currentrecvol;
					inR = psamples[i+1] * currentrecvol;
					if (inL > 32767.0f) { inL = 32767.0f; };
					if (inL < -32767.0f) { inL = -32767.0f; };
					if (inR > 32767.0f) { inR = 32767.0f; };
					if (inR < -32767.0f) { inR = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inL);
						currentnum++;
					}
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inR);
						currentnum++;
					}
					i++;
				};
				break;

			case 1:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i] * currentrecvol;
					inR = psamples[i+1] * currentrecvol;
					if (inL > 32767.0f) { inL = 32767.0f; };
					if (inL < -32767.0f) { inL = -32767.0f; };
					if (inR > 32767.0f) { inR = 32767.0f; };
					if (inR < -32767.0f) { inR = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inR);
						currentnum++;
					}
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inL);
						currentnum++;
					}
					i++;
				};
				break;
			case 2:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i] * currentrecvol;
					inR = psamples[i+1];
					if (inL > 32767.0f) { inL = 32767.0f; };
					if (inL < -32767.0f) { inL = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inL);
						currentnum++;
					}
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inL);
						currentnum++;
					}
					i++;
				};
				break;
			case 3:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i];
					inR = psamples[i+1] * currentrecvol;
					if (inR > 32767.0f) { inR = 32767.0f; };
					if (inR < -32767.0f) { inR = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inR);
						currentnum++;
					}
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inR);
						currentnum++;
					}
					i++;
				};
				break;
			case 4:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i] * currentrecvol;
					inR = psamples[i+1] * currentrecvol;
					if (inL > 32767.0f) { inL = 32767.0f; };
					if (inL < -32767.0f) { inL = -32767.0f; };
					if (inR > 32767.0f) { inR = 32767.0f; };
					if (inR < -32767.0f) { inR = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)((inL + inR) * 0.5f);
						currentnum++;
					}
					i++;
				};
				break;
			case 5:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i] * currentrecvol;
					inR = psamples[i+1];
					if (inL > 32767.0f) { inL = 32767.0f; };
					if (inL < -32767.0f) { inL = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inL);
						currentnum++;
					}
					i++;
				};
				break;
			case 6:
				for( i=0; i<numsamples*2; i++ ) {
					inL = psamples[i];
					inR = psamples[i+1] * currentrecvol;
					if (inR > 32767.0f) { inR = 32767.0f; };
					if (inR < -32767.0f) { inR = -32767.0f; };
					if (currentnum < currentnumofsamples) {
						u = currentnum;
						pLeva->pSamples[u] = (short)(inR);
						currentnum++;
					}
					i++;
				};
				break;
			default: break;
			};
		} // else mode
		if (currentnum > currentnumofsamples) {
			recordnow = 0;
			// correction Geoffroy 1/5/2003
			// fin de l'enregistrement, on peut remettre le gate
			if (aval.startGateOn == 1) {
				gateActive = 1;
				numsamplesabovegate = 0;
			}
			// fin correction Geoffroy 1/5/2003
		};
	};

	return false;
}

int mi::AllocateTrack(CSequence *pseq, int note)
{
	return -1;
}

void mi::MidiNoteOff(int c, CSequence *pseq, int notedelay)
{
	recordnow = 0;
	// correction Geoffroy 1/5/2003
	// fin de l'enregistrement, on peut remettre le gate
	if (aval.startGateOn == 1) {
		gateActive = 1;
		numsamplesabovegate = 0;
	}
	// fin correction Geoffroy 1/5/2003

	if (pseq != NULL)
	{
		byte *pdata = (byte *)pCB->GetPlayingRow(pseq, 1, 0);

		if (pdata[6] == SWITCH_NO) {
			pdata[6] = SWITCH_OFF;
		};
	};
};

void mi::MidiNote(int const channel, int const value, int const velocity)
{
	int wavebuffer;
	if (aval.midichannel == channel)
	{
		if (value / 12 > 9) return;
		if (value < 12) return;

		if (setmidion == 1) {
			aval.midinoteon = value;
			setmidioff = 0;
			setmidion = 0;
		} else if (setmidioff == 1) {
			aval.midinoteoff = value;
			setmidioff = 0;
			setmidion = 0;
		} else {
			CSequence *pseq;

			if (aval.midirecord == 1) {
				int stateflags = pCB->GetStateFlags();

				if (stateflags & SF_PLAYING && stateflags & SF_RECORDING)
					pseq = pCB->GetPlayingSequence(ThisMachine);
				else
					pseq = NULL;
			} else {
				pseq = NULL;
			};

			switch (aval.miditrigger) {
			case 0:
				// Don't do anything
				return;

				break;
			case 1:
			{
				// Midi On Sustain
				if (value == aval.midinoteon) {
					if (velocity > 0)
					{
						if (wavetouse != 0) {
							if (numofsamples > 0) {
								if (wavetouse == 201) {
									if (pCB->GetWave(199) != NULL) return;
									if (pCB->GetWave(200) != NULL) return;
									wavebuffer = pCB->GetFreeWave();
									if ((wavebuffer < 0) || (wavebuffer > 198)) return;
									waveinuse = wavebuffer;

								} else {
									waveinuse = wavetouse;
								};
								currentrecvol = recvol;
								currentinputmode = inputmode;
								if (aval.midimaxlen == 0) {
									currentnumofsamples = numofsamples;
								} else {
									numofsamples = aval.midimaxlen * pMasterInfo->SamplesPerTick;
								};
								currentnum = 0;
								recordnow = 1;
								if (aval.startGateOn == 1) gateActive = 1;

								switch (currentinputmode) {
								case 0:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 1:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (Inv. Chan.)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 2:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-L)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 3:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-R)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 4:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono") == true) {

									} else {
										recordnow = 0;
									};
									break;
								case 5:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-L)") == true) {

									} else {
										recordnow = 0;
									};
									break;
								case 6:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-R)") == true) {

									} else {
										recordnow = 0;
									};
									break;
								default:
									recordnow = 0;
									break;
								};
							};
						};

						if (aval.midirecord == 1) {
							if (pseq != NULL)
							{
								byte *pdata = (byte *)pCB->GetPlayingRow(pseq, 1, 0);
								pdata[6] = SWITCH_ON;
							};
						};
					}
					else
					{
						MidiNoteOff(0, pseq, 0);
					};
				};
				break;
			}
			case 2:
			{
				// Midi on / midi off
				if (value == aval.midinoteon) {
					if (velocity > 0)
					{
						if (wavetouse != 0) {
							if (numofsamples > 0) {
								if (wavetouse == 201) {
									if (pCB->GetWave(199) != NULL) return;
									if (pCB->GetWave(200) != NULL) return;
									wavebuffer = pCB->GetFreeWave();
									if ((wavebuffer < 0) || (wavebuffer > 198)) return;
									waveinuse = wavebuffer;

								} else {
									waveinuse = wavetouse;
								};
								currentrecvol = recvol;
								currentinputmode = inputmode;
								if (aval.midimaxlen == 0) {
									currentnumofsamples = numofsamples;
								} else {
									numofsamples = aval.midimaxlen * pMasterInfo->SamplesPerTick;
								};
								currentnum = 0;
								recordnow = 1;
								if (aval.startGateOn == 1) gateActive = 1;

								switch (currentinputmode) {
								case 0:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 1:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (Inv. Chan.)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 2:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-L)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 3:
									// Stereo Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples * 2, "Untitled Stereo (In-R)") == true) {
										currentnumofsamples *= 2;
										CWaveInfo const *mewav = pCB->GetWave(waveinuse);
										int *mewavefl = (int *)&mewav->Flags;
										*mewavefl = *mewavefl | WF_STEREO;

										CWaveLevel const *mewav2 = pCB->GetWaveLevel(waveinuse,0);
										int *mewavens = (int *)&mewav2->numSamples;
										*mewavens = currentnumofsamples / 2;
										int *mewavele = (int *)&mewav2->LoopEnd;
										*mewavele = currentnumofsamples / 2;

									} else {
										recordnow = 0;
									};
									break;
								case 4:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono") == true) {

									} else {
										recordnow = 0;
									};
									break;
								case 5:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-L)") == true) {

									} else {
										recordnow = 0;
									};
									break;
								case 6:
									// Mono Shit
									if (pCB->AllocateWave(waveinuse, currentnumofsamples, "Untitled Mono (In-R)") == true) {

									} else {
										recordnow = 0;
									};
									break;
								default:
									recordnow = 0;
									break;
								};
							};
						};

						if (aval.midirecord == 1) {
							if (pseq != NULL)
							{
								byte *pdata = (byte *)pCB->GetPlayingRow(pseq, 1, 0);
								pdata[6] = SWITCH_ON;
							};
						};
					}
				} else if (value == aval.midinoteoff) {
					if (velocity > 0)
					{
						MidiNoteOff(0, pseq, 0);
					}
				};
			}
			break;
			default: break;
			}
		};
	};
};


void mi::Command(int const i)
{
	switch (i)
	{
	case 0:
		setmidioff = 0;
		setmidion = 1;
		break;
	case 1:
		setmidioff = 1;
		setmidion = 0;
		break;
	case 2:
		MessageBox(NULL,"CnG Recorder 1.0\n\nCopyright 2001 Edward L. Blake\nDebugging and minor improvements in 2003 by Geoffroy\nEmail:cyanwerks.com/contact\nEmail:geoffroyATminizza.com\nUses the Utility Skin 2001 skin.","About CyanPhase Recorder",MB_OK|MB_SYSTEMMODAL);
		break;
	default:
		break;
	}
}
char const *mi::DescribeValue(int const param, int const value)
{
	static char txt[16];
	switch(param)
	{
	case 0:
		sprintf(txt,"%i ticks",value);
		return txt;
		break;
	case 1:
		if (value == 0) {
			return "Disabled";
		} else if (value == 201) {
			return "Automatic";
		} else {
			sprintf(txt,"Wave: %i",value);
			return txt;
		}
		break;
	case 2:
		switch (value) {

		case 0: return "Stereo"; break;
		case 1: return "Stereo Inv."; break;
		case 2: return "Stereo In-L"; break;
		case 3: return "Stereo In-R"; break;
		case 4: return "Mono"; break;
		case 5: return "Mono In-L"; break;
		case 6: return "Mono In-R"; break;
		default: return NULL; break;
		};
	case 3:
		sprintf(txt,"%.1f%%",value/127.0f * 100.0f);
		return txt;
		break;
	case 4:
		return NULL;
	default:
		return NULL;
		break;
	};
};

#pragma optimize ("", on)

DLL_EXPORTS

